@@ -1,8 +1,9 @@
 package Lemonldap::NG::Handler;
 
-use base 'Lemonldap::NG::Handler::ApacheMP2';
+print STDERR
+"Use the appropriate handler. For Apache, use Lemonldap::NG::Handler::ApacheMP2";
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 1;
 
@@ -36,7 +37,7 @@ L<http://lemonldap-ng.org/>
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
