@@ -11,7 +11,7 @@ use Lemonldap::NG::Common::UserAgent;
 use Lemonldap::NG::Common::FormEncode;
 use Lemonldap::NG::Common::Session;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 our @ISA     = ('Exporter');
 our @EXPORT  = qw(fetchId retrieveSession createSession hideCookie goToPortal);
 our @EXPORT_OK = @EXPORT;
@@ -24,8 +24,8 @@ our @EXPORT_OK = @EXPORT;
 # using indefinitely a session id disclosed accidentally or maliciously.
 # @return session id
 sub fetchId {
-    my $class = shift;
-    if ( my $creds = $class->header_in('Authorization') ) {
+    my ( $class, $req ) = @_;
+    if ( my $creds = $req->env->{'HTTP_AUTHORIZATION'} ) {
         $creds =~ s/^Basic\s+//;
         my @date = localtime;
         my $day  = $date[5] * 366 + $date[7];
@@ -41,17 +41,19 @@ sub fetchId {
 # and if needed, ask portal to create it through a SOAP request
 # @return true if the session was found, false else
 sub retrieveSession {
-    my ( $class, $id ) = @_;
+    my ( $class, $req, $id ) = @_;
 
     # First check if session already exists
-    if ( my $res = $class->Lemonldap::NG::Handler::Main::retrieveSession($id) )
+    if ( my $res =
+        $class->Lemonldap::NG::Handler::Main::retrieveSession( $req, $id ) )
     {
         return $res;
     }
 
     # Then ask portal to create it
-    if ( $class->createSession($id) ) {
-        return $class->Lemonldap::NG::Handler::Main::retrieveSession($id);
+    if ( $class->createSession( $req, $id ) ) {
+        return $class->Lemonldap::NG::Handler::Main::retrieveSession( $req,
+            $id );
     }
     else {
         return 0;
@@ -62,12 +64,12 @@ sub retrieveSession {
 # Ask portal to create it through a SOAP request
 # @return true if the session is created, else false
 sub createSession {
-    my ( $class, $id ) = @_;
+    my ( $class, $req, $id ) = @_;
 
     # Add client IP as X-Forwarded-For IP in SOAP request
-    my $xheader = $class->header_in('X-Forwarded-For');
+    my $xheader = $req->env->{'HTTP_X_FORWARDED_FOR'};
     $xheader .= ", " if ($xheader);
-    $xheader .= $class->remote_ip;
+    $xheader .= $req->{env}->{REMOTE_ADDR};
 
     #my $soapHeaders = HTTP::Headers->new( "X-Forwarded-For" => $xheader );
     ## TODO: use adminSession or sessions
@@ -76,7 +78,7 @@ sub createSession {
     #    default_headers => $soapHeaders
     #)->uri('urn:Lemonldap/NG/Common/PSGI/SOAPService');
 
-    my $creds = $class->header_in('Authorization');
+    my $creds = $req->env->{'HTTP_AUTHORIZATION'};
     $creds =~ s/^Basic\s+//;
     my ( $user, $pwd ) = ( decode_base64($creds) =~ /^(.*?):(.*)$/ );
     $class->logger->debug("AuthBasic authentication for user: $user");
@@ -84,18 +86,18 @@ sub createSession {
     #my $soapRequest = $soapClient->getCookies( $user, $pwd, $id );
     my $url = $class->tsv->{portal}->() . "/sessions/global/$id?auth";
     $url =~ s#//sessions/#/sessions/#g;
-    my $req = HTTP::Request->new( POST => $url );
-    $req->header( 'X-Forwarded-For' => $xheader );
-    $req->header( 'Content-Type'    => 'application/x-www-form-urlencoded' );
-    $req->header( Accept            => 'application/json' );
-    $req->content(
+    my $get = HTTP::Request->new( POST => $url );
+    $get->header( 'X-Forwarded-For' => $xheader );
+    $get->header( 'Content-Type'    => 'application/x-www-form-urlencoded' );
+    $get->header( Accept            => 'application/json' );
+    $get->content(
         build_urlencoded(
             user     => $user,
             password => $pwd,
             secret   => $class->tsv->{cipher}->encrypt(time)
         )
     );
-    my $resp = $class->ua->request($req);
+    my $resp = $class->ua->request($get);
 
     if ( $resp->is_success ) {
         $class->userLogger->notice("Good REST authentication for $user");
@@ -130,9 +132,9 @@ sub createSession {
 ## @rmethod protected void hideCookie()
 # Hide user credentials to the protected application
 sub hideCookie {
-    my $class = shift;
+    my ( $class, $req ) = @_;
     $class->logger->debug("removing Authorization header");
-    $class->unset_header_in('Authorization');
+    $class->unset_header_in( $req, 'Authorization' );
 }
 
 ## @rmethod protected int goToPortal(string url, string arg)
@@ -142,12 +144,13 @@ sub hideCookie {
 # @param $arg optionnal GET parameters
 # @return Apache2::Const::REDIRECT or Apache2::Const::AUTH_REQUIRED
 sub goToPortal {
-    my ( $class, $url, $arg ) = @_;
+    my ( $class, $req, $url, $arg ) = @_;
     if ($arg) {
-        return $class->Lemonldap::NG::Handler::Main::goToPortal( $url, $arg );
+        return $class->Lemonldap::NG::Handler::Main::goToPortal( $req, $url,
+            $arg );
     }
     else {
-        $class->set_header_out(
+        $class->set_header_out( $req,
             'WWW-Authenticate' => 'Basic realm="LemonLDAP::NG"' );
         return $class->AUTH_REQUIRED;
     }
