@@ -4,8 +4,22 @@ use strict;
 
 use Safe;
 use Lemonldap::NG::Common::Safelib;    #link protected safe Safe object
-use constant SAFEWRAP => ( Safe->can("wrap_code_ref") ? 1 : 0 );
-use Mouse;
+
+# Workaround for another ModPerl/Mouse issue...
+BEGIN {
+    require Mouse;
+    no warnings;
+    my $v = $Mouse::VERSION
+      ? sprintf( "%d.%03d%03d", ( $Mouse::VERSION =~ /(\d+)/g ) )
+      : 0;
+    if ( $v < 2.005001 and $Lemonldap::NG::Handler::Apache2::Main::VERSION ) {
+        require Moose;
+        Moose->import();
+    }
+    else {
+        Mouse->import();
+    }
+}
 
 has customFunctions => ( is => 'rw', isa => 'Maybe[Str]' );
 
@@ -15,25 +29,36 @@ has jail => ( is => 'rw' );
 
 has error => ( is => 'rw' );
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 ## @imethod protected build_jail()
 # Build and return the security jail used to compile rules and headers.
 # @return Safe object
 sub build_jail {
-    my ( $self, $api ) = @_;
+    my ( $self, $api, $require ) = @_;
 
     return $self->jail
       if (  $self->jail
         and $self->jail->useSafeJail
-        and $self->jail->customFunctions
         and $self->useSafeJail
-        and $self->customFunctions
-        and $self->jail->useSafeJail == $self->useSafeJail
-        and $self->jail->customFunctions == $self->customFunctions );
+        and $self->jail->useSafeJail == $self->useSafeJail );
 
     $self->useSafeJail(1) unless defined $self->useSafeJail;
 
+    if ($require) {
+        foreach my $f ( split /[, ]+/, $require ) {
+            if ( $f =~ /^[\w\:]+$/ ) {
+                eval "require $f";
+            }
+            else {
+                eval { require $f; };
+            }
+            if ($@) {
+                die "Unable to load '$f': $@";
+            }
+        }
+    }
+
     my @t =
       $self->customFunctions ? split( /\s+/, $self->customFunctions ) : ();
     foreach (@t) {
@@ -48,8 +73,7 @@ sub build_jail {
         }
         next if ( $self->can($_) );
         eval "sub $_ {
-            my \$uri = $api\::uri_with_args();
-            return $sub(\$uri,\@_)
+            return $sub(\@_)
         }";
         $api->logger->error($@) if ($@);
         $_ = "&$_";
@@ -66,16 +90,10 @@ sub build_jail {
     $self->jail->share_from( 'Lemonldap::NG::Common::Safelib',
         $Lemonldap::NG::Common::Safelib::functions );
 
-    $self->jail->share_from(
-        $api,
-        [
-            qw( &hostname &remote_ip &uri &uri_with_args
-              &unparsed_uri &args &method &header_in &env )
-        ]
-    );
     $self->jail->share_from( __PACKAGE__, [ @t, '&encrypt', '&token' ] );
     $self->jail->share_from( 'MIME::Base64', ['&encode_base64'] );
-    $self->jail->share_from( 'Lemonldap::NG::Handler::Main', ['$_v'] );
+
+    #$self->jail->share_from( 'Lemonldap::NG::Handler::Main', ['$_v'] );
 
     # Initialize cryptographic functions to be able to use them in jail.
     eval { token('a') };
@@ -142,25 +160,16 @@ sub share_from {
 }
 
 ## @imethod protected jail_reval()
-# Build and return restricted eval command with SAFEWRAP, if activated
+# Build and return restricted eval command
 # @return evaluation of $reval or $reval2
 sub jail_reval {
     my ( $self, $reval ) = @_;
 
-    $reval =~ s/\$ENV\{/&env()->\{/g;
-
     # if nothing is returned by reval, add the return statement to
     # the "no safe wrap" reval
 
     my $res;
-    eval {
-        $res = (
-            SAFEWRAP
-              and $self->useSafeJail
-            ? $self->jail->wrap_code_ref( $self->jail->reval($reval) )
-            : $self->jail->reval($reval)
-        );
-    };
+    eval { $res = ( $self->jail->reval($reval) ) };
     if ($@) {
         $self->error($@);
         return undef;
