@@ -1,7 +1,7 @@
 # Main running methods file
 package Lemonldap::NG::Handler::Main::Run;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 package Lemonldap::NG::Handler::Main;
 
@@ -23,37 +23,57 @@ sub handler {
 sub logout {
     my $class;
     $class = $#_ ? shift : __PACKAGE__;
-    $class->newRequest( $_[0] );
-    return $class->unlog();
+    return $class->unlog(@_);
 }
 
 sub status {
     my $class;
     $class = $#_ ? shift : __PACKAGE__;
-    $class->newRequest( $_[0] );
-    return $class->getStatus();
+    return $class->getStatus(@_);
 }
 
 # Public methods
 
 # Return Handler::Lib::Status output
 sub getStatus {
-    my ($class) = @_;
+    my ( $class, $req ) = @_;
     $class->logger->debug("Request for status");
     my $statusPipe = $class->tsv->{statusPipe};
     my $statusOut  = $class->tsv->{statusOut};
-    return $class->abort("$class: status page can not be displayed")
+    my $args       = '';
+    if ( $ENV{LLNGSTATUSHOST} ) {
+        require IO::Socket::INET;
+        foreach ( 64322 .. 64331 ) {
+            if ( $statusOut =
+                IO::Socket::INET->new( Proto => 'udp', LocalPort => $_ ) )
+            {
+                $args =
+                  ' host=' . ( $ENV{LLNGSTATUSCLIENT} || 'localhost' ) . ":$_";
+                last;
+            }
+        }
+        return $class->abort( $req,
+            "$class: status page can not be displayed, unable to open socket" )
+          unless ($statusOut);
+    }
+    return $class->abort( $req, "$class: status page can not be displayed" )
       unless ( $statusPipe and $statusOut );
-    print $statusPipe "STATUS"
-      . ( $class->args ? " " . $class->args : '' ) . "\n";
+    my $q = $req->{env}->{QUERY_STRING} || '';
+    if ( $q =~ /\s/ ) {
+        $class->logger->error("Bad characters in query");
+        return $class->FORBIDDEN;
+    }
+    $statusPipe->print(
+        "STATUS " . ( $req->{env}->{QUERY_STRING} || '' ) . "$args\n" );
     my $buf;
 
-    while (<$statusOut>) {
+    while ( $_ = $statusOut->getline ) {
         last if (/^END$/);
         $buf .= $_;
     }
-    $class->set_header_out( ( "Content-Type" => "text/html; charset=UTF-8" ) );
-    $class->print($buf);
+    $class->set_header_out( $req,
+        "Content-Type" => "text/html; charset=UTF-8" );
+    $class->print( $req, $buf );
     return $class->OK;
 }
 
@@ -62,12 +82,11 @@ sub getStatus {
 sub checkType {
     my ( $class, $req ) = @_;
 
-    $class->newRequest($req);
     if ( time() - $class->lastCheck > $class->checkTime ) {
         die("$class: No configuration found")
           unless ( $class->checkConf );
     }
-    my $vhost = $class->resolveAlias;
+    my $vhost = $class->resolveAlias($req);
     return ( defined $class->tsv->{type}->{$vhost} )
       ? $class->tsv->{type}->{$vhost}
       : 'Main';
@@ -84,14 +103,14 @@ sub run {
     my ( $class, $req, $rule, $protection ) = @_;
     my ( $id, $session );
 
-    return $class->DECLINED unless ( $class->is_initial_req );
+    return $class->DECLINED unless ( $class->is_initial_req($req) );
 
-    # Direct return if maintenance mode is active
-    if ( $class->checkMaintenanceMode ) {
+    # Direct return if maintenance mode is enabled
+    if ( $class->checkMaintenanceMode($req) ) {
 
         if ( $class->tsv->{useRedirectOnError} ) {
             $class->logger->debug("Go to portal with maintenance error code");
-            return $class->goToPortal( '/', 'lmError=' . $class->MAINTENANCE );
+            return $class->goToError( $req, '/', $class->MAINTENANCE );
         }
         else {
             $class->logger->debug("Return maintenance error code");
@@ -100,25 +119,25 @@ sub run {
     }
 
     # Cross domain authentication
-    my $uri = $class->unparsed_uri;
+    my $uri = $req->{env}->{REQUEST_URI};
 
-    $uri = $class->uri_with_args;
+    $uri = $req->{env}->{REQUEST_URI};
     my ($cond);
     ( $cond, $protection ) = $class->conditionSub($rule) if ($rule);
-    $protection = $class->isUnprotected($uri) || 0
+    $protection = $class->isUnprotected( $req, $uri ) || 0
       unless ( defined $protection );
 
     if ( $protection == $class->SKIP ) {
         $class->logger->debug("Access control skipped");
-        $class->updateStatus('SKIP');
-        $class->hideCookie;
-        $class->cleanHeaders;
+        $class->updateStatus( $req, 'SKIP' );
+        $class->hideCookie($req);
+        $class->cleanHeaders($req);
         return $class->OK;
     }
 
     # Try to recover cookie and user session
-    if (    $id = $class->fetchId
-        and $session = $class->retrieveSession($id) )
+    if (    $id = $class->fetchId($req)
+        and $session = $class->retrieveSession( $req, $id ) )
     {
 
         # AUTHENTICATION done
@@ -127,24 +146,25 @@ sub run {
         my $kc = keys %{$session};    # in order to detect new local macro
 
         # ACCOUNTING (1. Inform web server)
-        $class->set_user( $session->{ $class->tsv->{whatToTrace} } );
+        $class->set_user( $req, $session->{ $class->tsv->{whatToTrace} } );
 
         # AUTHORIZATION
-        return ( $class->forbidden($session), $session )
-          unless ( $class->grant( $session, $uri, $cond ) );
-        $class->updateStatus( 'OK', $session->{ $class->tsv->{whatToTrace} } );
+        return ( $class->forbidden( $req, $session ), $session )
+          unless ( $class->grant( $req, $session, $uri, $cond ) );
+        $class->updateStatus( $req, 'OK',
+            $session->{ $class->tsv->{whatToTrace} } );
 
         # ACCOUNTING (2. Inform remote application)
-        $class->sendHeaders($session);
+        $class->sendHeaders( $req, $session );
 
         # Store local macros
         if ( keys %$session > $kc ) {
             $class->logger->debug("Update local cache");
-            $class->session->update( $session, { updateCache => 2 } );
+            $req->data->{session}->update( $session, { updateCache => 2 } );
         }
 
         # Hide Lemonldap::NG cookie
-        $class->hideCookie;
+        $class->hideCookie($req);
 
         # Log access granted
         $class->logger->debug( "User "
@@ -152,8 +172,8 @@ sub run {
               . " was granted to access to $uri" );
 
         #  Catch POST rules
-        $class->postOutputFilter( $session, $uri );
-        $class->postInputFilter( $session, $uri );
+        $class->postOutputFilter( $req, $session, $uri );
+        $class->postInputFilter( $req, $session, $uri );
 
         return ( $class->OK, $session );
     }
@@ -162,9 +182,9 @@ sub run {
 
         # Ignore unprotected URIs
         $class->logger->debug("No valid session but unprotected access");
-        $class->updateStatus('UNPROTECT');
-        $class->hideCookie;
-        $class->cleanHeaders;
+        $class->updateStatus( $req, 'UNPROTECT' );
+        $class->hideCookie($req);
+        $class->cleanHeaders($req);
         return $class->OK;
     }
 
@@ -175,8 +195,8 @@ sub run {
           unless ($id);
 
         # if the cookie was fetched, a log is sent by retrieveSession()
-        $class->updateStatus( $id ? 'EXPIRED' : 'REDIRECT' );
-        return $class->goToPortal( $class->unparsed_uri );
+        $class->updateStatus( $req, $id ? 'EXPIRED' : 'REDIRECT' );
+        return $class->goToPortal( $req, $req->{env}->{REQUEST_URI} );
     }
 }
 
@@ -184,10 +204,10 @@ sub run {
 # Call localUnlog() then goToPortal() to unlog the current user.
 # @return Constant value returned by goToPortal()
 sub unlog {
-    my $class = shift;
-    $class->localUnlog(@_);
-    $class->updateStatus('LOGOUT');
-    return $class->goToPortal( '/', 'logout=1' );
+    my ( $class, $req ) = @_;
+    $class->localUnlog( $req, @_ );
+    $class->updateStatus( $req, 'LOGOUT' );
+    return $class->goToPortal( $req, '/', 'logout=1' );
 }
 
 # INTERNAL METHODS
@@ -198,12 +218,13 @@ sub unlog {
 # @param optional user string Username to log, if undefined defaults to remote IP
 # @param optional url string URL to log, if undefined defaults to request URI
 sub updateStatus {
-    my ( $class, $action, $user, $url ) = @_;
+    my ( $class, $req, $action, $user, $url ) = @_;
     my $statusPipe = $class->tsv->{statusPipe} or return;
-    $user ||= $class->remote_ip;
-    $url  ||= $class->uri_with_args;
+    $user ||= $req->{env}->{REMOTE_ADDR};
+    $url  ||= $req->{env}->{REQUEST_URI};
     eval {
-        print $statusPipe "$user => " . $class->hostname . "$url $action\n";
+        $statusPipe->print(
+            "$user => " . $req->{env}->{HTTP_HOST} . "$url $action\n" );
     };
 }
 
@@ -218,17 +239,17 @@ sub lmLog {
 
 ## @rmethod protected boolean checkMaintenanceMode
 # Check if we are in maintenance mode
-# @return true if maintenance mode
+# @return true if maintenance mode is enabled
 sub checkMaintenanceMode {
-    my $class = shift;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req ) = @_;
+    my $vhost = $class->resolveAlias($req);
     my $_maintenance =
       ( defined $class->tsv->{maintenance}->{$vhost} )
       ? $class->tsv->{maintenance}->{$vhost}
       : $class->tsv->{maintenance}->{_};
 
     if ($_maintenance) {
-        $class->logger->debug("Maintenance mode activated");
+        $class->logger->debug("Maintenance mode enabled");
         return 1;
     }
     return 0;
@@ -240,10 +261,16 @@ sub checkMaintenanceMode {
 # @param $cond optional Function granting access
 # @return True if the user is granted to access to the current URL
 sub grant {
-    my ( $class, $session, $uri, $cond, $vhost ) = @_;
-    return $cond->($session) if ($cond);
+    my ( $class, $req, $session, $uri, $cond, $vhost ) = @_;
+    return $cond->( $req, $session ) if ($cond);
 
-    $vhost ||= $class->resolveAlias;
+    $vhost ||= $class->resolveAlias($req);
+    if ( my $level = $class->tsv->{authnLevel}->{$vhost} ) {
+        if ( $session->{authenticationLevel} < $level ) {
+            $session->{_upgrade} = 1;
+            return 0;
+        }
+    }
     for (
         my $i = 0 ;
         $i < ( $class->tsv->{locationCount}->{$vhost} || 0 ) ;
@@ -254,7 +281,8 @@ sub grant {
             $class->logger->debug( 'Regexp "'
                   . $class->tsv->{locationConditionText}->{$vhost}->[$i]
                   . '" match' );
-            return $class->tsv->{locationCondition}->{$vhost}->[$i]->($session);
+            return $class->tsv->{locationCondition}->{$vhost}->[$i]
+              ->( $req, $session );
         }
     }
     unless ( $class->tsv->{defaultCondition}->{$vhost} ) {
@@ -264,37 +292,42 @@ sub grant {
         return 0;
     }
     $class->logger->debug("$vhost: Apply default rule");
-    return $class->tsv->{defaultCondition}->{$vhost}->($session);
+    return $class->tsv->{defaultCondition}->{$vhost}->( $req, $session );
 }
 
 ## @rmethod protected int forbidden(string uri)
 # Used to reject non authorized requests.
-# Inform the status processus and call logForbidden().
+# Inform the status process and call logForbidden().
 # @param $uri URI
 # @return Constant $class->FORBIDDEN
 sub forbidden {
-    my ( $class, $session, $vhost ) = @_;
-    my $uri = $class->unparsed_uri;
-    $vhost ||= $class->resolveAlias;
+    my ( $class, $req, $session, $vhost ) = @_;
+    my $uri = $req->{env}->{REQUEST_URI};
+    $vhost ||= $class->resolveAlias($req);
 
     if ( $session->{_logout} ) {
-        $class->updateStatus( 'LOGOUT',
+        $class->updateStatus( $req, 'LOGOUT',
             $session->{ $class->tsv->{whatToTrace} } );
         my $u = $session->{_logout};
-        $class->localUnlog;
-        return $class->goToPortal( $u, 'logout=1' );
+        $class->localUnlog($req);
+        return $class->goToPortal( $req, $u, 'logout=1' );
+    }
+
+    if ( $session->{_upgrade} ) {
+        return $class->goToPortal( $req, $uri, undef, '/upgradesession' );
     }
 
     # Log forbidding
     $class->userLogger->notice( "User "
           . $session->{ $class->tsv->{whatToTrace} }
           . " was forbidden to access to $vhost$uri" );
-    $class->updateStatus( 'REJECT', $session->{ $class->tsv->{whatToTrace} } );
+    $class->updateStatus( $req, 'REJECT',
+        $session->{ $class->tsv->{whatToTrace} } );
 
     # Redirect or Forbidden?
     if ( $class->tsv->{useRedirectOnForbidden} ) {
         $class->logger->debug("Use redirect for forbidden access");
-        return $class->goToPortal( $uri, 'lmError=403' );
+        return $class->goToError( $req, $uri, 403 );
     }
     else {
         $class->logger->debug("Return forbidden access");
@@ -305,25 +338,25 @@ sub forbidden {
 ## @rmethod protected void hideCookie()
 # Hide Lemonldap::NG cookie to the protected application.
 sub hideCookie {
-    my $class = shift;
+    my ( $class, $req ) = @_;
     $class->logger->debug("removing cookie");
-    my $cookie = $class->header_in('Cookie');
+    my $cookie = $req->env->{HTTP_COOKIE};
     my $cn     = $class->tsv->{cookieName};
     $cookie =~ s/$cn(http)?=[^,;]*[,;\s]*//og;
     if ($cookie) {
-        $class->set_header_in( 'Cookie' => $cookie );
+        $class->set_header_in( $req, 'Cookie' => $cookie );
     }
     else {
-        $class->unset_header_in('Cookie');
+        $class->unset_header_in( $req, 'Cookie' );
     }
 }
 
 ## @rmethod protected string encodeUrl(string url)
-# Encode URl in the format used by Lemonldap::NG::Portal for redirections.
+# Encode URL in the format used by Lemonldap::NG::Portal for redirections.
 # @return Base64 encoded string
 sub encodeUrl {
-    my ( $class, $url ) = @_;
-    $url = $class->_buildUrl($url) if ( $url !~ m#^https?://# );
+    my ( $class, $req, $url ) = @_;
+    $url = $class->_buildUrl( $req, $url ) if ( $url !~ m#^https?://# );
     return encode_base64( $url, '' );
 }
 
@@ -333,24 +366,38 @@ sub encodeUrl {
 # @param $arg optionnal GET parameters
 # @return Constant $class->REDIRECT
 sub goToPortal {
-    my ( $class, $url, $arg ) = @_;
+    my ( $class, $req, $url, $arg, $path ) = @_;
+    $path ||= '';
     my ( $ret, $msg );
-    my $urlc_init = $class->encodeUrl($url);
+    my $urlc_init = $class->encodeUrl( $req, $url );
     $class->logger->debug(
-        "Redirect " . $class->remote_ip . " to portal (url was $url)" );
-    $class->set_header_out( 'Location' => $class->tsv->{portal}->()
-          . "?url=$urlc_init"
+        "Redirect $req->{env}->{REMOTE_ADDR} to portal (url was $url)");
+    $class->set_header_out( $req,
+            'Location' => $class->tsv->{portal}->()
+          . "$path?url=$urlc_init"
           . ( $arg ? "&$arg" : "" ) );
     return $class->REDIRECT;
 }
 
+sub goToError {
+    my ( $class, $req, $url, $code ) = @_;
+    my $urlc_init = $class->encodeUrl( $req, $url );
+    $class->logger->debug(
+        "Redirect $req->{env}->{REMOTE_ADDR} to lmError (url was $url)");
+    $class->set_header_out( $req,
+            'Location' => $class->tsv->{portal}->()
+          . "/lmerror/$code"
+          . "?url=$urlc_init" );
+    return $class->REDIRECT;
+}
+
 ## @rmethod protected fetchId()
 # Get user cookies and search for Lemonldap::NG cookie.
 # @return Value of the cookie if found, 0 else
 sub fetchId {
-    my $class             = shift;
-    my $t                 = $class->header_in('Cookie') or return 0;
-    my $vhost             = $class->resolveAlias;
+    my ( $class, $req ) = @_;
+    my $t                 = $req->{env}->{HTTP_COOKIE} or return 0;
+    my $vhost             = $class->resolveAlias($req);
     my $lookForHttpCookie = (
         $class->tsv->{securedCookie} =~ /^(2|3)$/
           and !( defined( $class->tsv->{https}->{$vhost} ) )
@@ -381,21 +428,24 @@ sub fetchId {
 # Tries to retrieve the session whose index is id
 # @return true if the session was found, false else
 sub retrieveSession {
-    my ( $class, $id ) = @_;
+    my ( $class, $req, $id ) = @_;
     my $now = time();
 
     # 1. Search if the user was the same as previous (very efficient in
     # persistent connection).
-    if (    defined $class->datas->{_session_id}
-        and $id eq $class->datas->{_session_id}
-        and ( $now - $class->datasUpdate < 60 ) )
+    # NB: timout is here the same value as current HTTP/1.1 Keep-Alive timeout
+    #     (15 seconds)
+    if (    defined $class->data->{_session_id}
+        and $id eq $class->data->{_session_id}
+        and ( $now - $class->dataUpdate < $class->tsv->{handlerInternalCache} )
+      )
     {
         $class->logger->debug("Get session $id from Handler internal cache");
-        return $class->datas;
+        return $class->data;
     }
 
     # 2. Get the session from cache or backend
-    my $session = $class->session(
+    my $session = $req->data->{session} = (
         Lemonldap::NG::Common::Session->new(
             {
                 storageModule        => $class->tsv->{sessionStorageModule},
@@ -410,23 +460,46 @@ sub retrieveSession {
 
     unless ( $session->error ) {
 
-        $class->datas( $session->data );
-
-        $class->logger->debug("Get session $id");
+        $class->data( $session->data );
+        $class->logger->debug("Get session $id from Handler::Main::Run");
 
         # Verify that session is valid
+        $class->logger->error(
+"_utime is not defined. This should not happen. Check if it is well transmitted to handler"
+        ) unless $session->data->{_utime};
+
+        $class->logger->debug("Check session validity from Handler");
+        $class->logger->debug( "Session timeout -> " . $class->tsv->{timeout} );
+        $class->logger->debug( "Session timeoutActivity -> "
+              . $class->tsv->{timeoutActivity}
+              . "s" )
+          if ( $class->tsv->{timeoutActivity} );
+        $class->logger->debug(
+            "Session _utime -> " . $session->data->{_utime} );
+        $class->logger->debug( "now -> " . $now );
+        $class->logger->debug( "_lastSeen -> " . $session->data->{_lastSeen} )
+          if ( $session->data->{_lastSeen} );
+        my $delta = $now - $session->data->{_lastSeen}
+          if ( $session->data->{_lastSeen} );
+        $class->logger->debug( "now - _lastSeen = " . $delta )
+          if ( $session->data->{_lastSeen} );
+        $class->logger->debug( "Session timeoutActivityInterval -> "
+              . $class->tsv->{timeoutActivityInterval} )
+          if ( $class->tsv->{timeoutActivityInterval} );
+        my $ttl = $class->tsv->{timeout} - $now + $session->data->{_utime};
+        $class->logger->debug( "Session TTL = " . $ttl );
+
         if (
             $now - $session->data->{_utime} > $class->tsv->{timeout}
             or (    $class->tsv->{timeoutActivity}
                 and $session->data->{_lastSeen}
-                and $now - $session->data->{_lastSeen} >
-                $class->tsv->{timeoutActivity} )
+                and $delta > $class->tsv->{timeoutActivity} )
           )
         {
             $class->logger->info("Session $id expired");
 
             # Clean cached data
-            $class->datas( {} );
+            $class->data( {} );
             return 0;
         }
 
@@ -437,18 +510,19 @@ sub retrieveSession {
                 $class->tsv->{timeoutActivityInterval} )
           )
         {
-            $class->session->update( { '_lastSeen' => $now } );
+            $req->data->{session}->update( { '_lastSeen' => $now } );
+            $class->data( $session->data );
 
             if ( $session->error ) {
                 $class->logger->error("Cannot update session $id");
-                $class->logger->error( $class->session->error );
+                $class->logger->error( $req->data->{session}->error );
             }
             else {
                 $class->logger->debug("Update _lastSeen with $now");
             }
         }
 
-        $class->datasUpdate($now);
+        $class->dataUpdate($now);
         return $session->data;
     }
     else {
@@ -464,9 +538,10 @@ sub retrieveSession {
 # @param $s path
 # @return URL
 sub _buildUrl {
-    my ( $class, $s ) = @_;
-    my $vhost  = $class->hostname;
-    my $_https = (
+    my ( $class, $req, $s ) = @_;
+    my $realvhost = $req->{env}->{HTTP_HOST};
+    my $vhost     = $class->resolveAlias($req);
+    my $_https    = (
         defined( $class->tsv->{https}->{$vhost} )
         ? $class->tsv->{https}->{$vhost}
         : $class->tsv->{https}->{_}
@@ -474,12 +549,13 @@ sub _buildUrl {
     my $portString =
          $class->tsv->{port}->{$vhost}
       || $class->tsv->{port}->{_}
-      || $class->get_server_port;
+      || $req->{env}->{SERVER_PORT};
     $portString = (
-             ( $_https  && $portString == 443 )
+             ( $realvhost =~ /:\d+/ )
+          or ( $_https  && $portString == 443 )
           or ( !$_https && $portString == 80 )
     ) ? '' : ":$portString";
-    my $url = "http" . ( $_https ? "s" : "" ) . "://$vhost$portString$s";
+    my $url = "http" . ( $_https ? "s" : "" ) . "://$realvhost$portString$s";
     $class->logger->debug("Build URL $url");
     return $url;
 }
@@ -488,10 +564,10 @@ sub _buildUrl {
 # @param $uri URI
 # @return 0 if URI is protected,
 # $class->UNPROTECT if it is unprotected by "unprotect",
-# SKIP if is is unprotected by "skip"
+# SKIP if unprotected by "skip"
 sub isUnprotected {
-    my ( $class, $uri ) = @_;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req, $uri ) = @_;
+    my $vhost = $class->resolveAlias($req);
     for (
         my $i = 0 ;
         $i < ( $class->tsv->{locationCount}->{$vhost} || 0 ) ;
@@ -508,12 +584,13 @@ sub isUnprotected {
 ## @rmethod void sendHeaders()
 # Launch function compiled by forgeHeadersInit() for the current virtual host
 sub sendHeaders {
-    my ( $class, $session ) = @_;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req, $session ) = @_;
+    my $vhost = $class->resolveAlias($req);
     if ( defined $class->tsv->{forgeHeaders}->{$vhost} ) {
 
         # Log headers in debug mode
-        my %headers = $class->tsv->{forgeHeaders}->{$vhost}->($session);
+        my %headers =
+          $class->tsv->{forgeHeaders}->{$vhost}->( $req, $session );
         foreach my $h ( sort keys %headers ) {
             if ( defined( my $v = $headers{$h} ) ) {
                 $class->logger->debug("Send header $h with value $v");
@@ -522,26 +599,37 @@ sub sendHeaders {
                 $class->logger->debug("Send header $h with empty value");
             }
         }
-        $class->set_header_in(%headers);
+        $class->set_header_in( $req, %headers );
     }
 }
 
 ## @rmethod void cleanHeaders()
 # Unset HTTP headers, when sendHeaders is skipped
 sub cleanHeaders {
-    my $class = shift;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req ) = @_;
+    my $vhost = $class->resolveAlias($req);
     if ( defined( $class->tsv->{headerList}->{$vhost} ) ) {
-        $class->unset_header_in( @{ $class->tsv->{headerList}->{$vhost} } );
+        $class->unset_header_in( $req,
+            @{ $class->tsv->{headerList}->{$vhost} } );
     }
 }
 
 ## @rmethod string resolveAlias
 # returns vhost whose current hostname is an alias
 sub resolveAlias {
-    my $class = shift;
-    my $vhost = $class->hostname;
-    return $class->tsv->{vhostAlias}->{$vhost} || $vhost;
+    my ( $class, $req ) = @_;
+    my $vhost = ref $req ? $req->{env}->{HTTP_HOST} : $req;
+
+    $vhost =~ s/:\d+//;
+    return $class->tsv->{vhostAlias}->{$vhost}
+      if ( $class->tsv->{vhostAlias}->{$vhost} );
+    return $vhost if ( $class->tsv->{defaultCondition}->{$vhost} );
+    my $v = $vhost;
+    while ( $v =~ s/[\w\-]+/\*/ ) {
+        return $v if ( $class->tsv->{defaultCondition}->{$v} );
+        $v =~ s/^\*\.*//;
+    }
+    return $vhost;
 }
 
 #__END__
@@ -552,18 +640,18 @@ sub resolveAlias {
 # @param $msg Message to log
 # @return Constant ($class->REDIRECT, $class->SERVER_ERROR)
 sub abort {
-    my ( $class, $msg ) = @_;
+    my ( $class, $req, $msg ) = @_;
 
     # If abort is called without a valid request, fall to die
     eval {
-        my $uri = $class->unparsed_uri;
+        my $uri = $req->{env}->{REQUEST_URI};
 
         $class->logger->error($msg);
 
         # Redirect or die
         if ( $class->tsv->{useRedirectOnError} ) {
             $class->logger->debug("Use redirect for error");
-            return $class->goToPortal( $uri, 'lmError=500' );
+            return $class->goToError( $req, $uri, 500 );
         }
         else {
             return $class->SERVER_ERROR;
@@ -575,16 +663,17 @@ sub abort {
 ## @rmethod protected void localUnlog()
 # Delete current user from local cache entry.
 sub localUnlog {
-    my ( $class, $id ) = @_;
+    my ( $class, $req, $id ) = @_;
     $class->logger->debug('Local handler logout');
-    if ( $id //= $class->fetchId ) {
+    if ( $id //= $class->fetchId($req) ) {
 
-        # Delete thread datas
-        if (    $class->datas->{_session_id}
-            and $id eq $class->datas->{_session_id} )
+        # Delete thread data
+        if (    $class->data->{_session_id}
+            and $id eq $class->data->{_session_id} )
         {
-            $class->datas( {} );
+            $class->data( {} );
         }
+        delete $req->data->{session};
 
         # Delete local cache
         if (    $class->tsv->{refLocalStorage}
@@ -599,36 +688,37 @@ sub localUnlog {
 # Add a javascript to html page in order to fill html form with fake data
 # @param uri URI to catch
 sub postOutputFilter {
-    my ( $class, $session, $uri ) = @_;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req, $session, $uri ) = @_;
+    my $vhost = $class->resolveAlias($req);
 
     if ( defined( $class->tsv->{outputPostData}->{$vhost}->{$uri} ) ) {
         $class->logger->debug("Filling a html form with fake data");
 
-        $class->unset_header_in("Accept-Encoding");
+        $class->unset_header_in( $req, "Accept-Encoding" );
         my %postdata =
-          $class->tsv->{outputPostData}->{$vhost}->{$uri}->($session);
+          $class->tsv->{outputPostData}->{$vhost}->{$uri}->( $req, $session );
         my $formParams = $class->tsv->{postFormParams}->{$vhost}->{$uri};
-        my $js = $class->postJavascript( \%postdata, $formParams );
-        $class->addToHtmlHead($js);
+        my $js = $class->postJavascript( $req, \%postdata, $formParams );
+        $class->addToHtmlHead( $req, $js );
     }
 }
 
 ## @rmethod protected postInputFilter(string uri)
-# Replace request body with form datas defined in configuration
+# Replace request body with form data defined in configuration
 # @param uri URI to catch
 sub postInputFilter {
-    my ( $class, $session, $uri ) = @_;
-    my $vhost = $class->resolveAlias;
+    my ( $class, $req, $session, $uri ) = @_;
+    my $vhost = $class->resolveAlias($req);
 
     if ( defined( $class->tsv->{inputPostData}->{$vhost}->{$uri} ) ) {
         $class->logger->debug("Replacing fake data with real form data");
 
-        my %data = $class->tsv->{inputPostData}->{$vhost}->{$uri}->($session);
+        my %data =
+          $class->tsv->{inputPostData}->{$vhost}->{$uri}->( $req, $session );
         foreach ( keys %data ) {
             $data{$_} = uri_escape( $data{$_} );
         }
-        $class->setPostParams( \%data );
+        $class->setPostParams( $req, \%data );
     }
 }
 
@@ -636,7 +726,7 @@ sub postInputFilter {
 # build a javascript to fill a html form with fake data
 # @param data hashref containing input => value
 sub postJavascript {
-    my ( $class, $data, $formParams ) = @_;
+    my ( $class, $req, $data, $formParams ) = @_;
 
     my $form = $formParams->{formSelector} || "form";
 
@@ -645,7 +735,7 @@ sub postJavascript {
         use bytes;
         my $value = "x" x bytes::length( $data->{$name} );
         $filler .=
-"form.find('input[name=$name], select[name=$name], textarea[name=$name]').val('$value')\n";
+"form.find('input[name=\"$name\"], select[name=\"$name\"], textarea[name=\"$name\"]').val('$value')\n";
     }
 
     my $submitter =
