@@ -9,7 +9,7 @@ use Lemonldap::NG::Handler::PSGI::Main;
 
 extends 'Lemonldap::NG::Handler::Lib::PSGI', 'Lemonldap::NG::Common::PSGI';
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 sub init {
     my ( $self, $args ) = @_;
@@ -92,7 +92,7 @@ Level of protection. It can be one of:
 
 =head3 user
 
-Returns user session datas. If empty (no protection), returns:
+Returns user session data. If empty (no protection), returns:
 
   { _whatToTrace => 'anonymous' }
 
@@ -125,7 +125,7 @@ L<Lemonldap::NG::Common::PSGI::Request>, L<HTML::Template>,
 =head1 BUG REPORT
 
 Use OW2 system to report bug or ask for features:
-L<http://jira.ow2.org>
+L<https://gitlab.ow2.org/lemonldap-ng/lemonldap-ng/issues>
 
 =head1 DOWNLOAD
 
